"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Spider = void 0;
const request_1 = __importDefault(require("request"));
const libxmljs_1 = __importDefault(require("libxmljs"));
const Mp3_1 = __importDefault(require("./Mp3"));
class Spider {
    constructor(options = {}) {
        this.options = options;
    }
    static create(options) {
        return new Spider(options);
    }
    _doGet(url) {
        return new Promise((resolve, reject) => {
            request_1.default({ method: "GET", uri: url }, (err, res, body) => {
                if (err) {
                    reject(err);
                    return;
                }
                if (res.statusCode !== 200) {
                    reject("Request Failed: " + url + " - " + res.statusCode);
                    return;
                }
                resolve(body);
            });
        });
    }
    getLinks(url) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const body = yield this._doGet(url);
                return libxmljs_1.default
                    .parseHtmlString(body)
                    .find("//a")
                    .map((v) => { var _a; return (_a = v.attr("href")) === null || _a === void 0 ? void 0 : _a.value(); })
                    .filter((v_1) => v_1 !== null &&
                    v_1 !== undefined &&
                    v_1 !== "/" &&
                    v_1.slice(0, 4) !== "http" &&
                    url.indexOf(v_1) < 0);
            }
            catch (err) {
                if (this.options.onError) {
                    return this.options.onError(err);
                }
                return [];
            }
        });
    }
    forMp3s(url, siteMap = { root: "", mp3s: [], paths: {} }, allMp3s = []) {
        return __awaiter(this, void 0, void 0, function* () {
            siteMap.root = url;
            const parts = yield this.getLinks(url);
            yield Promise.all(parts.map((part) => {
                if (part.slice(part.length - 4) === ".mp3") {
                    const mp3 = Mp3_1.default.create(url + part);
                    siteMap.mp3s.push(mp3);
                    allMp3s.push(mp3);
                    return [siteMap, allMp3s];
                }
                else {
                    const newMap = { root: "", mp3s: [], paths: {} };
                    siteMap.paths[part] = newMap;
                    return new Promise((resolve) => setTimeout(() => resolve(this.forMp3s(url + part, newMap, allMp3s)), 0));
                }
            }));
            return [siteMap, allMp3s];
        });
    }
}
exports.Spider = Spider;
//# sourceMappingURL=Spider.js.map